@@ -2,6 +2,10 @@
 # pylint: disable=too-many-locals
 # pylint: disable=too-many-arguments
 # pylint: disable=too-many-statements
+# pylint: disable=too-many-lines
+# pylint: disable=protected-access
+# pylint: disable=consider-using-enumerate
+
 """
 A module for finding the beam size in an monochrome image.
 
@@ -9,35 +13,55 @@ Simple and fast calculation of beam sizes from a single monochrome image based
 on the ISO 11146 method of variances.  Some effort has been made to make
 the algorithm less sensitive to background offset and noise.
 
-Finding the center and dimensions of a monochrome image of a beam is simple::
+Finding the center and diameters of a beam in a monochrome image is simple::
 
     import imageio
     import numpy as np
     import laserbeamsize as lbs
 
-    beam = imageio.imread("t-hene.pgm")
-    x, y, dx, dy, phi = lbs.beam_size(beam)
+    beam_image = imageio.imread("t-hene.pgm")
+    x, y, dx, dy, phi = lbs.beam_size(beam_image)
 
-    print("The center of the beam ellipse is at (%.0f, %.0f)" % (x,y))
+    print("The center of the beam ellipse is at (%.0f, %.0f)" % (x, y))
     print("The ellipse diameter (closest to horizontal) is %.0f pixels" % dx)
     print("The ellipse diameter (closest to   vertical) is %.0f pixels" % dy)
     print("The ellipse is rotated %.0f° ccw from the horizontal" % (phi*180/3.1416))
 
-    ellipticity = dy/dx
-    if ellipticity>0.87:
-        print()
-        print("The beam is circular because the ellipticity=%.2f > 0.87")
-        print("The circular beam diameter is %.0f pixels" % np.sqrt((dx**2+dy**2)/2))
+A full graphic can be created by::
+
+    lbs.beam_size_plot(beam_image)
+    plt.show()
+
+A mosaic of images might be created by::
+
+    # read images for each location
+    z = np.array([89,94,99,104,109,114,119,124,129,134,139], dtype=float) #[mm]
+    filenames = ["%d.pgm" % location for location in z]
+    images = [imageio.imread(filename) for filename in filenames]
+
+    lbs.beam_size_montage(images, z*1e-3, pixel_size=3.75, crop=True)
+    plt.show()
 """
 
-import matplotlib.pyplot as plt
 import numpy as np
+import matplotlib.pyplot as plt
+import matplotlib._cm
+import matplotlib.cm
+import matplotlib.colors
 import scipy.ndimage
 
+# cubeYF palette described at https://mycarta.wordpress.com
+# the default gist_ncar colormap works better for most beams
+specs = matplotlib._cm.cubehelix(gamma=1.4, s=0.4, r=-0.8, h=2.0)
+cubeYF_cmap = matplotlib.colors.LinearSegmentedColormap('cubeYF', specs)
+matplotlib.cm.register_cmap(cmap=cubeYF_cmap)
+
 __all__ = ('subtract_image',
            'subtract_threshold',
+           'subtract_tilted_background',
            'corner_background',
            'corner_mask',
+           'perimeter_mask',
            'corner_subtract',
            'rotate_image',
            'rotated_rect_mask',
@@ -50,16 +74,17 @@ __all__ = ('subtract_image',
            'draw_beam_figure',
            'ellipse_arrays',
            'elliptical_mask',
-           'plot_image_and_ellipse',
            'major_axis_arrays',
            'minor_axis_arrays',
-           'visual_report',
+           'beam_size_plot',
+           'beam_size_and_plot',
+           'beam_size_montage'
            )
 
 
 def rotate_points(x, y, x0, y0, phi):
     """
-    Rotate x and y around designated center (x0,y0).
+    Rotate x and y around designated center (x0, y0).
 
     Args:
         x: x-values of point or array of points to be rotated
@@ -69,7 +94,7 @@ def rotate_points(x, y, x0, y0, phi):
         phi: angle to rotate (+ is ccw) in radians
 
     Returns:
-        x,y locations of rotated points
+        x, y locations of rotated points
     """
     xp = x-x0
     yp = y-y0
@@ -86,15 +111,15 @@ def rotate_points(x, y, x0, y0, phi):
     return xf, yf
 
 
-def values_along_line(image, x0, y0, x1, y1, N=50):
+def values_along_line(image, x0, y0, x1, y1, N=100):
     """
-    Return x, y, z, and distance values between (x0,y0) and (x1,y1).
+    Return x, y, z, and distance values between (x0, y0) and (x1, y1).
 
     This creates four arrays::
         x: index of horizontal pixel values along line
         y: index of vertical pixel values along line
-        z: image values at each of the x,y positions
-        s: distance from start of minor axis to x,y position
+        z: image values at each of the x, y positions
+        s: distance from start of minor axis to x, y position
 
     Args:
         image: the image to work with
@@ -128,8 +153,8 @@ def major_axis_arrays(image, xc, yc, dx, dy, phi, diameters=3):
     This creates four arrays::
         x: index of horizontal pixel values along line
         y: index of vertical pixel values along line
-        z: image values at each of the x,y positions
-        s: distance from start of minor axis to x,y position
+        z: image values at each of the x, y positions
+        s: distance from start of minor axis to x, y position
 
     Args:
         image: the image to work with
@@ -169,8 +194,8 @@ def minor_axis_arrays(image, xc, yc, dx, dy, phi, diameters=3):
     This creates four arrays::
         x: index of horizontal pixel values along line
         y: index of vertical pixel values along line
-        z: image values at each of the x,y positions
-        s: distance from start of minor axis to x,y position
+        z: image values at each of the x, y positions
+        s: distance from start of minor axis to x, y position
 
     Args:
         image: the image to work with
@@ -275,7 +300,7 @@ def rotate_image(original, x0, y0, phi):
     # center of rotated image, defaults mode='constant' and cval=0.0
     ry, rx = (np.array(rotated.shape)-1)/2.0
 
-    # position of (x0,y0) in rotated image
+    # position of (x0, y0) in rotated image
     new_x0, new_y0 = rotate_points(x0, y0, cx, cy, phi)
     new_x0 += rx - cx
     new_y0 += ry - cy
@@ -316,9 +341,9 @@ def basic_beam_size(image):
 
     The returned parameters are::
 
-        `xc`,`yc` is the center of the elliptical spot.
+        `xc`, `yc` is the center of the elliptical spot.
 
-        `dx`,`dy` is the semi-major/minor diameters of the elliptical spot.
+        `dx`, `dy` is the semi-major/minor diameters of the elliptical spot.
 
         `phi` is tilt of the ellipse from the axis [radians]
 
@@ -422,6 +447,29 @@ def corner_mask(image, corner_fraction=0.035):
     the_mask[-n:, -m:] = True
     return the_mask
 
+def perimeter_mask(image, corner_fraction=0.035):
+    """
+    Create boolean mask for image with a perimeter marked as True.
+
+    The perimeter is the same width as the corners created by corner_mask.
+
+    Args:
+        image : the image to work with
+        corner_fraction: determines the width of the perimeter
+    Returns:
+        boolean 2D array with corners marked True
+    """
+    v, h = image.shape
+    n = int(v * corner_fraction)
+    m = int(h * corner_fraction)
+
+    the_mask = np.full_like(image, False, dtype=np.bool)
+    the_mask[:, :m] = True
+    the_mask[:, -m:] = True
+    the_mask[:n, :] = True
+    the_mask[-n:, :] = True
+    return the_mask
+
 
 def corner_background(image, corner_fraction=0.035):
     """
@@ -453,7 +501,7 @@ def corner_subtract(image, corner_fraction=0.035, nT=3):
     """
     Return image with background subtracted.
 
-    The background is estimated using the values in the cornes of the
+    The background is estimated using the values in the corners of the
     image.  The new image will have a constant (`mean+nT*stdev`) subtracted.
 
     ISO 11146-3 recommends values from 2-5% for `corner_fraction`.
@@ -473,6 +521,49 @@ def corner_subtract(image, corner_fraction=0.035, nT=3):
     offset = int(back + nT * sigma)
     return subtract_threshold(image, offset)
 
+def subtract_tilted_background(image, corner_fraction=0.035):
+    """
+    Return image with tilted planar background subtracted.
+
+    Take all the points around the perimeter of an image and fit these
+    to a tilted plane to determine the background to subtract.  Details of
+    the linear algebra are at https://math.stackexchange.com/questions/99299
+
+    Since the sample contains noise, it is important not to remove
+    this noise at this stage and therefore we offset the plane so
+    that one standard deviation of noise remains.
+
+    Args:
+        image : the image to work with
+        corner_fraction: the fractional size of corner rectangles
+    Returns:
+        new image with tilted planar background subtracted
+    """
+    v, h = image.shape
+    xx, yy = np.meshgrid(range(h), range(v))
+
+    mask = perimeter_mask(image, corner_fraction=corner_fraction)
+    perimeter_values = image[mask]
+    # coords is (y_value,x_value,1) for each point in perimeter_values
+    coords = np.stack((yy[mask], xx[mask], np.ones(np.size(perimeter_values))), 1)
+
+    # fit a plane to all corner points
+    b = np.matrix(perimeter_values).T
+    A = np.matrix(coords)
+    fit = (A.T * A).I * A.T * b
+    # find coefficients of fit
+    a = fit[0][0, 0]
+    b = fit[1][0, 0] # overwriting b
+    c = fit[2][0, 0]
+    # calculate the fit plane
+    z = a * yy + b*xx + c
+    # find the standard deviation of the noise in the perimeter
+    # and subtract this value from the plane,
+    # since we don't want to lose the image noise just yet
+    z -= np.std(perimeter_values)
+
+    # finally, subtract the plane from the original image
+    return subtract_image(image, z)
 
 def rotated_rect_mask(image, xc, yc, dx, dy, phi, mask_diameters=3):
     """
@@ -510,7 +601,7 @@ def rotated_rect_mask(image, xc, yc, dx, dy, phi, mask_diameters=3):
 
 def rotated_rect_arrays(xc, yc, dx, dy, phi, mask_diameters=3):
     """
-    Return x,y arrays to draw a rotated rectangle.
+    Return x, y arrays to draw a rotated rectangle.
 
     Args:
         xc: horizontal center of beam
@@ -520,23 +611,23 @@ def rotated_rect_arrays(xc, yc, dx, dy, phi, mask_diameters=3):
         phi: angle that elliptical beam is rotated [radians]
 
     Returns:
-        x,y : two arrays for points on corners of rotated rectangle
+        x, y : two arrays for points on corners of rotated rectangle
     """
     rx = mask_diameters * dx / 2
     ry = mask_diameters * dy / 2
 
-    # rectangle with center at (xc,yc)
+    # rectangle with center at (xc, yc)
     x = np.array([-rx, -rx, +rx, +rx, -rx]) + xc
     y = np.array([-ry, +ry, +ry, -ry, -ry]) + yc
 
     x_rot, y_rot = rotate_points(x, y, xc, yc, phi)
 
-    return x_rot, y_rot
+    return np.array([x_rot, y_rot])
 
 
 def axes_arrays(xc, yc, dx, dy, phi, mask_diameters=3):
     """
-    Return x,y arrays needed to draw semi-axes of ellipse.
+    Return x, y arrays needed to draw semi-axes of ellipse.
 
     Args:
         xc: horizontal center of beam
@@ -546,18 +637,18 @@ def axes_arrays(xc, yc, dx, dy, phi, mask_diameters=3):
         phi: angle that elliptical beam is rotated [radians]
 
     Returns:
-        x,y arrays needed to draw semi-axes of ellipse
+        x, y arrays needed to draw semi-axes of ellipse
     """
     rx = mask_diameters * dx / 2
     ry = mask_diameters * dy / 2
 
-    # major and minor ellipse axes with center at (xc,yc)
+    # major and minor ellipse axes with center at (xc, yc)
     x = np.array([-rx, rx, 0, 0, 0]) + xc
     y = np.array([0, 0, 0, -ry, ry]) + yc
 
     x_rot, y_rot = rotate_points(x, y, xc, yc, phi)
 
-    return x_rot, y_rot
+    return np.array([x_rot, y_rot])
 
 
 def beam_size(image, mask_diameters=3, corner_fraction=0.035, nT=3, max_iter=25):
@@ -590,9 +681,9 @@ def beam_size(image, mask_diameters=3, corner_fraction=0.035, nT=3, max_iter=25)
 
     The returned parameters are::
 
-        `xc`,`yc` is the center of the elliptical spot.
+        `xc`, `yc` is the center of the elliptical spot.
 
-        `dx`,`dy` are the diameters of the elliptical spot.
+        `dx`, `dy` are the diameters of the elliptical spot.
 
         `phi` is tilt of the ellipse from the axis [radians]
 
@@ -675,7 +766,7 @@ def beam_test_image(h, v, xc, yc, dx, dy, phi, noise=0, max_value=255):
 
 def ellipse_arrays(xc, yc, dx, dy, phi, npoints=200):
     """
-    Return x,y arrays to draw a rotated ellipse.
+    Return x, y arrays to draw a rotated ellipse.
 
     Args:
         xc: horizontal center of beam
@@ -684,49 +775,14 @@ def ellipse_arrays(xc, yc, dx, dy, phi, npoints=200):
         dy: vertical diameter of beam
         phi: angle that elliptical beam is rotated [radians]
     Returns:
-        x,y : two arrays of points on the ellipse
+        x, y : two arrays of points on the ellipse
     """
     t = np.linspace(0, 2*np.pi, npoints)
     a = dx/2*np.cos(t)
     b = dy/2*np.sin(t)
     xp = xc + a*np.cos(phi) - b*np.sin(phi)
     yp = yc - a*np.sin(phi) - b*np.cos(phi)
-    return xp, yp
-
-
-def plot_image_and_ellipse(image, xc, yc, dx, dy, phi, scale=1):
-    """
-    Plot the image, an ellipse, and center lines.
-
-    Args:
-        image: 2D array of image with beam spot
-        xc: horizontal center of beam
-        yc: vertical center of beam
-        dx: horizontal diameter of beam
-        dy: vertical diameter of beam
-        phi: angle that elliptical beam is rotated [radians]
-        scale: factor to increase/decrease ellipse size
-    """
-    v, h = image.shape
-    xp, yp = ellipse_arrays(xc, yc, dx, dy, phi)
-    xp *= scale
-    yp *= scale
-    xcc = xc * scale
-    ycc = yc * scale
-    dxx = dx * scale
-    dyy = dy * scale
-    ph = phi * 180/np.pi
-
-    # show the beam image with actual dimensions on the axes
-    plt.imshow(image, extent=[0, h*scale, v*scale, 0], cmap='gray')
-    plt.plot(xp, yp, ':y')
-    plt.plot([xcc, xcc], [0, v*scale], ':y')
-    plt.plot([0, h*scale], [ycc, ycc], ':y')
-    plt.title(r'c=(%.0f,%.0f), (dx,dy)=(%.1f,%.1f), $\phi$=%.1f°' %
-              (xcc, ycc, dxx, dyy, ph))
-    plt.xlim(0, h*scale)
-    plt.ylim(v*scale, 0)
-    plt.colorbar()
+    return np.array([xp, yp])
 
 
 def basic_beam_size_naive(image):
@@ -810,7 +866,7 @@ def draw_beam_figure():
 
     plt.annotate(r'$\phi$', xy=(13, -2.5), fontsize=16)
     plt.annotate('', xy=(15.5, 0), xytext=(
-        14, -8.0), arrowprops=dict(arrowstyle="<-", connectionstyle="arc3,rad=-0.2"))
+        14, -8.0), arrowprops=dict(arrowstyle="<-", connectionstyle="arc3, rad=-0.2"))
 
     plt.annotate(r'$d_x$', xy=(-17, 7), color='blue', fontsize=16)
     plt.annotate(r'$d_y$', xy=(-4, -8), color='red', fontsize=16)
@@ -820,84 +876,436 @@ def draw_beam_figure():
     plt.axis('off')
 
 
-def visual_report(image, title='Original', **kwargs):
+def crop_image_to_rect(image, xc, yc, xmin, xmax, ymin, ymax):
     """
-    Create a visual report for image fitting.
+    Return image cropped to specified rectangle.
 
     Args:
-        image: 2D image of laser beam
-        title: optional title for report
+        image: image of beam
+        xc,yc: beam center (pixels)
+        xmin: left edge (pixels)
+        xmax: right edge (pixels)
+        ymin: top edge (pixels)
+        ymax: bottom edge (pixels)
     Returns:
-        nothing
+        cropped_image: cropped image
+        new_xc, new_yc: new beam center (pixels)
     """
+    v, h = image.shape
+    xmin = max(0, int(xmin))
+    xmax = min(h, int(xmax))
+    ymin = max(0, int(ymin))
+    ymax = min(v, int(ymax))
+    new_xc = xc-xmin
+    new_yc = yc-ymin
+    return image[ymin:ymax, xmin:xmax], new_xc, new_yc
+
+
+def crop_image_to_integration_rect(image, xc, yc, dx, dy, phi):
+    """
+    Return image cropped to integration rectangle.
+
+    Since the image is being cropped, the center of the beam will move.
+
+    Args:
+        image: image of beam
+        xc: horizontal center of beam
+        yc: vertical center of beam
+        dx: horizontal diameter of beam
+        dy: vertical diameter of beam
+        phi: angle that elliptical beam is rotated [radians]
+    Returns:
+        cropped_image: cropped image
+        new_xc: x-position of beam center in cropped image
+        new_yc: y-position of beam center in cropped image
+    """
+    xp, yp = rotated_rect_arrays(xc, yc, dx, dy, phi, mask_diameters=3)
+    return crop_image_to_rect(image, xc, yc, min(xp), max(xp), min(yp), max(yp))
+
+def luminance(value, cmap_name='gist_ncar', vmin=0, vmax=255):
+    """Return luminance of depending on cmap and value."""
+    # value between 0 and 1
+    v = (value-vmin)/(vmax-vmin)
+    v = min(max(0, v), 1)
+    cmap = matplotlib.cm.get_cmap(cmap_name)
+
+    # 0.3 seems like a reasonable compromise
+    rgb = cmap(v)
+    r = 255 * rgb[0]
+    g = 255 * rgb[1]
+    b = 255 * rgb[2]
+    lum = 0.2126 * r + 0.7152 * g + 0.0722 * b # per ITU-R BT.709
+    return lum
+
+def draw_as_dotted_contrast_line(image, xpts, ypts, cmap='gist_ncar', vmax=None):
+    """Draw lines in white or black depending on background image."""
+    if vmax is None:
+        vmax = np.max(image)
+
+    v, h = image.shape
+
+    # find the luminance at each point
+    lumas = np.array([], dtype=float)
+    for k in range(len(xpts)):
+        i = int(ypts[k])
+        j = int(xpts[k])
+        if 0 <= i < v and 0 <= j < h:
+            luma = luminance(image[i, j], cmap_name=cmap, vmax=vmax)
+            lumas = np.append(lumas, luma)
+
+    if len(lumas) == 0 or np.min(lumas) > 40:
+        contrast_color = "black"
+    else:
+        contrast_color = "white"
+
+    # draw the points
+    plt.plot(xpts, ypts, ':', color=contrast_color)
+
+def beam_size_and_plot(o_image,
+                       pixel_size=None,
+                       vmax=None,
+                       units='µm',
+                       crop=False,
+                       colorbar=False,
+                       cmap='gist_ncar',
+                       **kwargs):
+    """
+    Plot the image, fitted ellipse, integration area, and semi-major/minor axes.
+
+    If pixel_size is defined, then the returned measurements are in units of
+    pixel_size.
+
+    This is helpful for creating a mosaics of all the images created for an
+    experiment.
+
+    If `crop` is a two parameter list `[v,h]` then `v` and `h` are
+    interpreted as the vertical and horizontal sizes of the rectangle.  The
+    size is in pixels unless `pixel_size` is specified.  In that case the
+    rectangle sizes are in whatever units `pixel_size` is .
+
+    If `crop==True` then the displayed image is cropped to the ISO 11146 integration
+    rectangle.
+
+    All cropping is done after analysis and therefosre only affects
+    what is displayed.  If the image needs to be cropped before analysis
+    then that must be done before calling this function.
+
+    Args:
+        image: 2D array of image with beam spot
+        pixel_size: (optional) size of pixels
+        vmax: (optional) maximum value for colorbar
+        units: (optional) string used for units used on axes
+        crop: (optional) crop image to integration rectangle
+
+    Returns:
+        xc: horizontal center of beam
+        yc: vertical center of beam
+        dx: horizontal diameter of beam
+        dy: vertical diameter of beam
+        phi: angle that elliptical beam is rotated [radians]
+    """
+    # only pass along arguments that apply to beam_size()
     beamsize_keys = ['mask_diameters', 'corner_fraction', 'nT', 'max_iter']
     bs_args = dict((k, kwargs[k]) for k in beamsize_keys if k in kwargs)
 
-    corner_keys = ['corner_fraction', 'nT']
-    c_args = dict((k, kwargs[k]) for k in corner_keys if k in kwargs)
+    # find center and diameters
+    xc, yc, dx, dy, phi = beam_size(o_image, **bs_args)
 
-    xc, yc, dx, dy, phi = beam_size(image, **bs_args)
+    # establish scale and correct label
+    if pixel_size is None:
+        scale = 1
+        label = 'Pixels'
+    else:
+        scale = pixel_size
+        label = 'Position (%s)' % units
+
+    # crop image if necessary
+    if isinstance(crop, list):
+        ymin = yc-crop[0]/2/scale # in pixels
+        ymax = yc+crop[0]/2/scale
+        xmin = xc-crop[1]/2/scale
+        xmax = xc+crop[1]/2/scale
+        image, xc, yc = crop_image_to_rect(o_image, xc, yc, xmin, xmax, ymin, ymax)
+    elif crop:
+        image, xc, yc = crop_image_to_integration_rect(o_image, xc, yc, dx, dy, phi)
+    else:
+        image = o_image
+
+    # establish maximum colorbar value
+    if vmax is None:
+        vmax = image.max()
+
+    # extents may be changed by scale
+    v, h = image.shape
+    extent = np.array([-xc, h-xc, v-yc, -yc])*scale
+
+    # display image and axes labels
+    im = plt.imshow(image, extent=extent, cmap=cmap, vmax=vmax)
+    plt.xlabel(label)
+    plt.ylabel(label)
+
+    # draw semi-major and semi-minor axes
+    xp, yp = axes_arrays(xc, yc, dx, dy, phi)
+    draw_as_dotted_contrast_line(image, (xp-xc)*scale, (yp-yc)*scale, vmax=vmax, cmap=cmap)
+
+    # show ellipse around beam
+    xp, yp = ellipse_arrays(xc, yc, dx, dy, phi)
+    draw_as_dotted_contrast_line(image, (xp-xc)*scale, (yp-yc)*scale, vmax=vmax, cmap=cmap)
+
+    # show integration area around beam
+    xp, yp = rotated_rect_arrays(xc, yc, dx, dy, phi)
+    draw_as_dotted_contrast_line(image, (xp-xc)*scale, (yp-yc)*scale, vmax=vmax, cmap=cmap)
+
+    # set limits on axes
+    plt.xlim(-xc*scale, (h-xc)*scale)
+    plt.ylim((v-yc)*scale, -yc*scale)
+
+    # show colorbar
+    if colorbar:
+        v, h = image.shape
+        plt.colorbar(im, fraction=0.046*v/h, pad=0.04)
+
+    return xc*scale, yc*scale, dx*scale, dy*scale, phi
+
+
+def beam_size_plot(o_image,
+                   title='Original',
+                   pixel_size=None,
+                   units='µm',
+                   crop=False,
+                   cmap='gist_ncar',
+                   **kwargs):
+    """
+    Create a visual report for image fitting.
+
+    If `crop` is a two parameter list `[v,h]` then `v` and `h` are
+    interpreted as the vertical and horizontal sizes of the rectangle.  The
+    size is in pixels unless `pixel_size` is specified.  In that case the
+    rectangle sizes are in whatever units `pixel_size` is .
+
+    If `crop==True` then the displayed image is cropped to the ISO 11146 integration
+    rectangle.
+
+    All cropping is done after analysis and therefosre only affects
+    what is displayed.  If the image needs to be cropped before analysis
+    then that must be done before calling this function.
+
+    Args:
+        image: 2D image of laser beam
+        title: optional title for upper left plot
+        pixel_size: (optional) size of pixels
+        units: (optional) string used for units used on axes
+        crop: (optional) crop image to integration rectangle
+    Returns:
+        nothing
+    """
+    # only pass along arguments that apply to beam_size()
+    beamsize_keys = ['mask_diameters', 'corner_fraction', 'nT', 'max_iter']
+    bs_args = dict((k, kwargs[k]) for k in beamsize_keys if k in kwargs)
+
+    # find center and diameters
+    xc, yc, dx, dy, phi = beam_size(o_image, **bs_args)
+
+    # determine scaling and labels
+    if pixel_size is None:
+        scale = 1
+        unit_str = ''
+        units = 'pixels'
+        label = 'Pixels from Center'
+    else:
+        scale = pixel_size
+        unit_str = '[%s]' % units
+        label = 'Distance from Center %s' % unit_str
+
+    # crop image as appropriate
+    if isinstance(crop, list):
+        ymin = yc-crop[0]/2/scale # in pixels
+        ymax = yc+crop[0]/2/scale
+        xmin = xc-crop[1]/2/scale
+        xmax = xc+crop[1]/2/scale
+        image, xc, yc = crop_image_to_rect(o_image, xc, yc, xmin, xmax, ymin, ymax)
+    elif crop:
+        image, xc, yc = crop_image_to_integration_rect(o_image, xc, yc, dx, dy, phi)
+    else:
+        image = o_image
+
+    # subtract background based on corners
+    c_args = dict((k, kwargs[k]) for k in ['corner_fraction', 'nT'] if k in kwargs)
     working_image = corner_subtract(image, **c_args)
-    min_ = image.min()
-    max_ = image.max()
+
+    # get background value to use when calculating fitted beam
+    cb_args = dict((k, kwargs[k]) for k in ['corner_fraction'] if k in kwargs)
+    background, _ = corner_background(image, **cb_args)
+
     rx = dx/2
     ry = dy/2
+
+    min_ = image.min()
+    max_ = image.max()
     vv, hh = image.shape
 
+    # scale all the dimensions
+    v_s = vv * scale
+    h_s = hh * scale
+    xc_s = xc * scale
+    yc_s = yc * scale
+    dx_s = dx * scale
+    dy_s = dy * scale
+
     plt.subplots(2, 2, figsize=(12, 12))
     plt.subplots_adjust(right=1.0)
 
     # original image
     plt.subplot(2, 2, 1)
-    plt.imshow(image, cmap='gist_ncar')
-    plt.title(title)
+    im = plt.imshow(image, cmap=cmap)
+    plt.colorbar(im, fraction=0.046*v_s/h_s, pad=0.04)
     plt.clim(min_, max_)
-    plt.colorbar(fraction=0.046, pad=0.04)
-    plt.xlabel('Pixels')
-    plt.ylabel('Pixels')
-    plt.ylim(vv, 0)
-    plt.xlim(0, hh)
+    plt.xlabel('Position (pixels)')
+    plt.ylabel('Position (pixels)')
+    plt.title(title)
 
     # working image
     plt.subplot(2, 2, 2)
-    plt.imshow(working_image, cmap='gist_ncar')
-    xp, yp = axes_arrays(xc, yc, dx, dy, phi)
-    plt.plot(xp, yp, ':y')
-    xp, yp = rotated_rect_arrays(xc, yc, dx, dy, phi)
-    plt.plot(xp, yp, 'w')
-
-    plt.title('Found (%d, %d), dx=%d, dy=%d' % (xc, yc, dx, dy))
-    plt.clim(min_, max_)
-    plt.colorbar(fraction=0.046, pad=0.04)
-    plt.xlabel('Pixels')
-    plt.ylabel('Pixels')
-    plt.ylim(vv, 0)
-    plt.xlim(0, hh)
+    extent = np.array([-xc_s, h_s-xc_s, v_s-yc_s, -yc_s])
+    im = plt.imshow(working_image, extent=extent, cmap=cmap)
+    xp, yp = ellipse_arrays(xc, yc, dx, dy, phi) * scale
+    draw_as_dotted_contrast_line(working_image, xp-xc_s, yp-yc_s)
+    xp, yp = axes_arrays(xc, yc, dx, dy, phi) * scale
+    draw_as_dotted_contrast_line(working_image, xp-xc_s, yp-yc_s)
+    xp, yp = rotated_rect_arrays(xc, yc, dx, dy, phi) * scale
+    draw_as_dotted_contrast_line(working_image, xp-xc_s, yp-yc_s)
+    plt.colorbar(im, fraction=0.046*v_s/h_s, pad=0.04)
+#    plt.clim(min_, max_)
+    plt.xlim(-xc_s, h_s-xc_s)
+    plt.ylim(v_s-yc_s, -yc_s)
+    plt.xlabel(label)
+    plt.ylabel(label)
+    plt.title('Image w/o background, center at (%.0f, %.0f) %s' % (xc_s, yc_s, units))
 
     # plot of values along semi-major axis
-    plt.subplot(2, 2, 3)
     _, _, z, s = major_axis_arrays(image, xc, yc, dx, dy, phi)
-    a = np.sqrt(2/np.pi)/rx * np.sum(z)*abs(s[1]-s[0])
-    h = a*np.exp(-2)
-    plt.plot(s, z, 'or', label='data')
-    plt.plot(s, a*np.exp(-2*(s/rx)**2), ':k', label='fitted')
-    plt.xlabel('Distance from Center (pixels)')
+    a = np.sqrt(2/np.pi)/rx * np.sum(z-background)*abs(s[1]-s[0])
+    baseline = a*np.exp(-2)+background
+
+    plt.subplot(2, 2, 3)
+    plt.plot(s*scale, z, 'b')
+    plt.plot(s*scale, a*np.exp(-2*(s/rx)**2)+background, ':k')
+    plt.annotate('', (-rx*scale, baseline), (rx*scale, baseline), arrowprops=dict(arrowstyle="<->"))
+    plt.text(0, 1.1*baseline, 'dx=%.0f %s' % (dx_s, units), va='bottom', ha='center')
+    plt.text(0, a, '  Gaussian')
+    plt.xlabel('Distance from Center [%s]' % units)
     plt.ylabel('Pixel Intensity Along Semi-Major Axis')
     plt.title('Semi-Major Axis')
-    plt.annotate('', (-rx, h), (rx, h), arrowprops=dict(arrowstyle="<->"))
-    plt.text(0, 1.1*h, 'dx=%.1f' % dx, va='bottom', ha='center')
-    plt.text(0, a, '  Gaussian')
+    plt.gca().set_ylim(bottom=0)
 
     # plot of values along semi-minor axis
-    plt.subplot(2, 2, 4)
     _, _, z, s = minor_axis_arrays(image, xc, yc, dx, dy, phi)
-    a = np.sqrt(2/np.pi)/ry * np.sum(z)*abs(s[1]-s[0])
-    h = a*np.exp(-2)
-    plt.plot(s, z, 'or', label='data')
-    plt.plot(s, a*np.exp(-2*(s/ry)**2), ':k', label='fitted')
-    plt.xlabel('Distance from Center (pixels)')
+    a = np.sqrt(2/np.pi)/ry * np.sum(z-background)*abs(s[1]-s[0])
+    baseline = a*np.exp(-2)+background
+
+    plt.subplot(2, 2, 4)
+    plt.plot(s*scale, z, 'b')
+    plt.plot(s*scale, a*np.exp(-2*(s/ry)**2)+background, ':k', label='fitted')
+    plt.annotate('', (-ry*scale, baseline), (ry*scale, baseline), arrowprops=dict(arrowstyle="<->"))
+    plt.text(0, 1.1*baseline, 'dy=%.0f %s' % (dy_s, units), va='bottom', ha='center')
+    plt.text(0, a, '  Gaussian')
+    plt.xlabel('Distance from Center [%s]' % units)
     plt.ylabel('Pixel Intensity Along Semi-Minor Axis')
     plt.title('Semi-Minor Axis')
-    plt.annotate('', (-ry, h), (ry, h), arrowprops=dict(arrowstyle="<->"))
-    plt.text(0, 1.1*h, 'dy=%.1f' % dy, va='bottom', ha='center')
-    plt.text(0, a, '  Gaussian')
+    plt.gca().set_ylim(bottom=0)
+
+    # add more horizontal space between plots
+    plt.subplots_adjust(wspace=0.3)
+
+def beam_size_montage(images,
+                      z=None,
+                      cols=3,
+                      pixel_size=None,
+                      vmax=None,
+                      units='µm',
+                      crop=False,
+                      cmap='gist_ncar',
+                      **kwargs):
+    """
+    Create a beam size montage for a set of images.
+
+    If `crop` is a two parameter list `[v,h]` then `v` and `h` are
+    interpreted as the vertical and horizontal sizes of the rectangle.  The
+    size is in pixels unless `pixel_size` is specified.  In that case the
+    rectangle sizes are in whatever units `pixel_size` is .
+
+    If `crop==True` then the displayed image is cropped to the ISO 11146 integration
+    rectangle.
+
+    All cropping is done after analysis and therefosre only affects
+    what is displayed.  If the image needs to be cropped before analysis
+    then that must be done before calling this function.
+
+    Args:
+        images: array of 2D images of the laser beam
+        z: (optional) array of axial positions of images (always in meters!)
+        cols: (optional) number of columns in the montage
+        pixel_size: (optional) size of pixels
+        units: (optional) string used for units used on axes
+        crop: (optional) crop image to integration rectangle
+    Returns:
+        dx,dy: semi-major and semi-minor diameters
+    """
+    # arrays to save diameters
+    dx = np.zeros(len(images))
+    dy = np.zeros(len(images))
+
+    # calculate the number of rows needed in the montage
+    rows = (len(images)-1) // cols + 1
+
+    # when pixel_size is not specified, units default to pixels
+    if pixel_size is None:
+        units = 'pixels'
+
+    # gather all the options that are fixed for every image in the montage
+    options = {'pixel_size':pixel_size,
+               'vmax':vmax,
+               'units':units,
+               'crop':crop,
+               'cmap':cmap,
+               **kwargs}
+
+    # now set up the grid of subplots
+    plt.subplots(rows, cols, figsize=(cols*5, rows*5))
+
+    for i, im in enumerate(images):
+        plt.subplot(rows, cols, i+1)
+
+        # should we add color bar?
+        cb = not (vmax is None) and (i+1 == cols)
+
+        # plot the image and gather the beam diameters
+        _, _, dx[i], dy[i], _ = beam_size_and_plot(im, **options, colorbar=cb)
+
+        # add a title
+        if units == 'mm':
+            s = "dx=%.2f%s, dy=%.2f%s" % (dx[i], units, dy[i], units)
+        else:
+            s = "dx=%.0f%s, dy=%.0f%s" % (dx[i], units, dy[i], units)
+        if z is None:
+            plt.title(s)
+        else:
+            plt.title("z=%.0fmm, %s" % (z[i]*1e3, s))
+
+        # omit y-labels on all but first column
+        if i%cols:
+            plt.ylabel("")
+            if isinstance(crop, list):
+                plt.yticks([])
+
+        # omit x-labels on all but last row
+        if i < (rows-1)*cols:
+            plt.xlabel("")
+            if isinstance(crop, list):
+                plt.xticks([])
+
+    for i in range(len(images), rows*cols):
+        plt.subplot(rows, cols, i+1)
+        plt.axis("off")
+
+    return dx, dy
